# -*- coding: utf-8 -*-
# https://helpx.adobe.com/extension-manager/using/extension-manager-error-code-reference.html
import os
import re
import shutil
import subprocess
import sys
from time import sleep

isZH = True
baseDir = ""
ExManCmd = r"/Volumes/cmdline64/Contents/MacOS/ExManCmd"
bundleId = "com.idealabui"
extensionDir1 = r"/Library/Application Support/Adobe/CEP/extensions/%s" % bundleId
extensionDir2 = r"~/Library/Application Support/Adobe/CEP/extensions/%s" % bundleId

ExManCmdDmg = ""
pythonVersion = 2

zxpPath = ""


def processing_flush(n, index=5):
    n = int(n)
    n_ = int(n * index / 100) % index
    # sys.stdout.write("\Downloading 100%%")
    if isZH:
        sys.stdout.write("\r下载中 %s%s %03s%%" %
                         ((n_) * ".", (index - 1 - (n_)) * " ", n))
    else:
        sys.stdout.write("\rDownloading %s%s %03s%%" %
                         ((n_) * ".", (index - 1 - (n_)) * " ", n))
    # sys.stdout.write("\rDownloading %s" % ((n % index) * "."))
    sys.stdout.flush()


def show_progress(block_num, block_size, total_size):

    downloaded = block_num * block_size

    percent = downloaded*100.0/total_size
    processing_flush(percent, index=20)


def run_cmd(cmd):
    sub = subprocess.Popen(cmd, shell=True,
                           stdout=subprocess.PIPE)
    sub.wait()
    return sub.stdout.read().strip().decode('utf-8')


def showDialog(title, message, isSuccess=True):

    cmd = """osascript -e 'display alert "%s" message "%s"  as %s ' """ % (
        title, message, "informational" if isSuccess else "critical")

    run_cmd(cmd)
    if not isSuccess:
        sys.exit(1)


def showDialogWithButton(title, message):
    confirmButton = "Confirm"
    cancelButton = "Cancel"
    if isZH:
        confirmButton = "OK"
        cancelButton = "取消"

    cmd = """osascript -e 'display alert "%s" message "%s" as informational buttons {"%s", "%s"} default button "%s" cancel button "%s" ' """ % (
        title, message, cancelButton, confirmButton, confirmButton, cancelButton)

    result = run_cmd(cmd)
    prefix = "button returned:"
    if not prefix.startswith(prefix):
        if isZH:
            cmd = """ osascript -e 'display alert "Install Failed" message "Run Command Failed" as critical ' """
        else:
            cmd = """ osascript -e 'display alert "安装失败" message "运行命令失败" as critical ' """
        result = run_cmd(cmd)
        sys.exit(1)
    result = result[len(prefix):]

    if result != confirmButton:
        if isZH:
            showDialog("Versebook插件安装已取消",
                       "用户取消安装", isSuccess=False)
        else:
            showDialog("Versebook Plugin Installation Canceled",
                       "Installation was canceled by the user.", isSuccess=False)
        sys.exit(1)

    # print(result)


def checkMd5():
    result = run_cmd('md5 "%s"' % ExManCmdDmg)
    return result.endswith(
        "87fa903dbeb7f6096f7e1c152e71d7d1")


def getExManCmd():
    needDownloadExManCmd = False
    if not os.path.isfile(ExManCmd):
        needDownloadExManCmd = True
        ExManCmdDmgExist = False
        if os.path.isfile(ExManCmdDmg):

            ExManCmdDmgExist = checkMd5()
            if not ExManCmdDmgExist:
                os.remove(ExManCmdDmg)

        if not ExManCmdDmgExist:

            if int(pythonVersion) == 2:
                import urllib as request
            else:
                import urllib.request as request

            if isZH:
                showDialogWithButton(
                    "Versebook插件安装提示", "现在即将去Adobe官网下载Exman安装工具")
            else:
                showDialogWithButton(
                    "Versebook Plugin Install", "The installation tool Exman will be downloaded from the Adobe website.")

            url = "https://download.macromedia.com/pub/extensionmanager/ExManCmd_mac.dmg"

            try:
                os.makedirs(os.path.dirname(ExManCmdDmg))
            except:
                pass

            if isZH:
                print("\r开始将 %s 下载到 %s\r" % (url, ExManCmdDmg))
            else:
                print("\rdownloading %s to %s\r" % (url, ExManCmdDmg))

            request.urlretrieve(url, ExManCmdDmg, show_progress)

        run_cmd("hdiutil attach %s" % ExManCmdDmg)

    if not os.path.isfile(ExManCmd):
        if isZH:
            showDialog("Versebook插件安装失败", "下载ExManCmd失败", isSuccess=False)
        else:
            showDialog("Versebook Plugin Install Failed",
                       "Download ExManCmd failed", isSuccess=False)
        sys.exit(1)

    if needDownloadExManCmd:
        if isZH:
            print("\r下载ExMan完成\r")
        else:
            print("\rExMan Downloaded\r")


def is_installed():
    result = run_cmd('"%s" --list all' % (ExManCmd))
    return "installed for" in result and "com.idealabui" in result


def uninstall_failed():
    if isZH:
        showDialog("Versebook 插件安装失败", "卸载旧版本插件失败", isSuccess=False)
    else:
        showDialog("Versebook Plugin Install Failed",
                   "Uninstall previous plugin failed", isSuccess=False)


def uninstall_plugin():
    if isZH:
        print("\r开始卸载旧版本插件...\r")
    else:
        print("\rUninstalling previous plugin...\r")
    count = 0
    while True:
        count = count + 1
        if count > 4:
            uninstall_failed()
            sys.exit(1)

        if is_installed():
            run_cmd('"%s" --remove "%s"' % (ExManCmd, bundleId))
            pass
        else:
            sleep(1)
            break

    if is_installed():
        uninstall_failed()
        sys.exit(1)

    try:
        shutil.rmtree(extensionDir1)
    except:
        pass

    try:
        shutil.rmtree(extensionDir2)
    except:
        pass


def install_plugin_by_upa():

    UnifiedPluginInstallerAgentCmd = r"/Library/Application Support/Adobe/Adobe Desktop Common/RemoteComponents/UPI/UnifiedPluginInstallerAgent/UnifiedPluginInstallerAgent.app/Contents/MacOS/UnifiedPluginInstallerAgent"

    UnifiedPluginInstallerAgentCmd2 = r"~/Library/Application Support/Adobe/Adobe Desktop Common/RemoteComponents/UPI/UnifiedPluginInstallerAgent/UnifiedPluginInstallerAgent.app/Contents/MacOS/UnifiedPluginInstallerAgent"
    if not os.path.isfile(UnifiedPluginInstallerAgentCmd):
        UnifiedPluginInstallerAgentCmd = UnifiedPluginInstallerAgentCmd2
        if not os.path.isfile(UnifiedPluginInstallerAgentCmd):
            return False

    result = run_cmd('"%s" --install "%s"' %
                     (UnifiedPluginInstallerAgentCmd, zxpPath))
    if "Installation Successful for extension with file path" in result and is_installed():
        if isZH:
            showDialog("Versebook 插件安装成功",
                       "Versebook Adobe插件安装成功", isSuccess=True)
        else:
            showDialog("Versebook Plugin Install Success",
                       "The Versebook plugin has been installed to Adobe Creative Cloud", isSuccess=True)
        sys.exit(0)


def install_plugin():
    if isZH:
        print("\r开始安装插件...\r")
    else:
        print("\rInstalling plugin...\r")
    result = run_cmd('"%s" --install "%s"' % (ExManCmd, zxpPath))
    if "Installation Successful for extension with file path" in result and is_installed():
        if isZH:
            showDialog("Versebook 插件安装成功",
                       "Versebook Adobe插件安装成功", isSuccess=True)
        else:
            showDialog("Versebook Plugin Install Success",
                       "The Versebook plugin has been installed to Adobe Creative Cloud", isSuccess=True)
        sys.exit(0)

    if "-175" in result:
        if isZH:
            # https://helpx.adobe.com/extension-manager/using/extension-manager-error-code-reference.html
            result = "您需要正确输入当前管理员用户密码来进行安装."
        else:
            result = "You need to correctly enter the current admin user password to install."
    elif "-193" in result:
        if isZH:
            # https://helpx.adobe.com/extension-manager/using/extension-manager-error-code-reference.html
            result = "请确认是否正确安装了Adobe Creative Cloud."
        else:
            result = "Please confirm that Adobe Creative Cloud is installed correctly."

    if True:  # not install_plugin_by_upa():
        if isZH:
            showDialog("Versebook 插件安装失败", "Versebook Adobe插件安装失败:\n\n%s" %
                       result, isSuccess=True)
        else:
            showDialog("Versebook Plugin Install Failed",
                       "Failed to install the Versebook plugin to Adobe Creative Cloud:\n\n%s" %
                       result, isSuccess=True)
        sys.exit(-1)


if __name__ == '__main__':
    if len(sys.argv) > 1:
        isZH = sys.argv[1] == 'zh'

    if isZH:
        showDialogWithButton(
            "Versebook 插件安装提示", "即将为您安装插件, 请确认您已安装了Adobe Creative Cloud\n\n安装过程中可能需要多次输入macOS当前用户密码,请按提示输入")
    else:
        showDialogWithButton(
            "Versebook Plugin Install", "Before installing the plugin, please make sure you have Adobe Creative Cloud installed.\n\nDuring the installation process, you may need to enter the current macOS user password multiple times, please enter it as prompted")

    pythonVersion = sys.version_info[0]
    print("Using python%s" % pythonVersion)

    baseDir = os.path.dirname(os.path.abspath(__file__))
    baseDir = os.path.join(baseDir, "..")
    baseDir = os.path.abspath(baseDir)

    zxpPath = os.path.join(baseDir, "CEP", "idealab.zxp")
    ExManCmdDmg = os.path.join(baseDir, "download", "ExManCmd_mac.dmg")

    if not os.path.isfile(zxpPath):
        if isZH:
            showDialog("Versebook插件安装失败", "插件文件缺失", isSuccess=False)
        else:
            showDialog("Versebook Plugin Install Failed",
                       "Plugin file missing", isSuccess=False)
        sys.exit(1)

    getExManCmd()

    uninstall_plugin()
    install_plugin()
